package analyzer;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.springframework.util.StringUtils;

import pt.uc.dei.wsdbench.sut.domain.schema.Main;
import pt.uc.dei.wsdbench.sut.domain.schema.helper.Helper;
import pt.uc.dei.wsdbench.sut.domain.schema.robustness.Dependency;
import pt.uc.dei.wsdbench.sut.domain.schema.robustness.Param;
import pt.uc.dei.wsdbench.sut.domain.schema.type.ComplexType;
import pt.uc.dei.wsdbench.sut.external.LongWrapper;
import pt.uc.dei.wsdbench.sut.injector.PersistenceException;
import pt.uc.dei.wsdbench.sut.injector.configuration.Configuration;
import pt.uc.dei.wsdbench.sut.injector.persistence.PersistenceManager;
import pt.uc.dei.wsdbench.sut.injector.result.PendingResult;
import pt.uc.dei.wsdbench.sut.injector.tools.Tools;
import pt.uc.dei.wsdbench.sut.service.input.NewCustomerInput;

public class Initializer {

    private PersistenceManager<PendingResult> persistenceManager = new PersistenceManager<PendingResult>();
    private List<PendingResult> pendingResultList;
    private Map<String, String[]> classNameToXsdFileMap = new HashMap<String, String[]>();
    public static final String OUT_DEC = "ChangePaymentMethodOutputDeclaration.xsd";
    public static final String OUT_DEF = "ChangePaymentMethodOutputDefinition.xsd";
    private static final int TOTAL_FILES = 80;
    private static final String IN_DEC = "ChangePaymentMethodInputDeclaration.xsd";
    private static final String IN_DEF = "ChangePaymentMethodInputDefinition.xsd";

    private void init(String fileName) throws PersistenceException {
        classNameToXsdFileMap.put(LongWrapper.class.getName(), new String[]{"LongWrapperDeclaration.xsd", "LongWrapperDefinition.xsd"});

        System.out.println("--------------------BEGINNING-----------------------");

        classNameToXsdFileMap.put(NewCustomerInput.class.getName(), new String[]{IN_DEC, IN_DEF});

        classNameToXsdFileMap.put(NewCustomerInput.class.getName(), new String[]{OUT_DEC, OUT_DEF});

        pendingResultList = persistenceManager.loadPendingResultList(fileName, TOTAL_FILES);

        Cleaner.removeNoFaultRequests(pendingResultList);
        System.out.println("Requests with faults: " + pendingResultList.size());
        Cleaner.removeValidDomainOutputs(pendingResultList);
        System.out.println("->->->->->->->->->Requests with faults and invalid final responses (valid domain responses were removed): " + pendingResultList.size());

        Map<String, List<String>> classNameAndproblematicFieldsMap = new HashMap<String, List<String>>();
        int i = 0;
        for (PendingResult pendingResult : pendingResultList) {
            System.out.println(++i);
            String[] xsd = classNameToXsdFileMap.get(pendingResult.getInput().getClass().getName());

            String field;

            field = StringUtils.unqualify(pendingResult.getFullFieldName());

            List<String> probFields = classNameAndproblematicFieldsMap.get(pendingResult.getInput().getClass().getName());
            if (probFields == null) {
                probFields = new ArrayList<String>();
                probFields.add(field);
                classNameAndproblematicFieldsMap.put(pendingResult.getInput().getClass().getName(), probFields);
            } else {
                probFields.add(field);
            }

            System.out.println("****SOME PROBLEMATIC FIELD******");
            System.out.println(pendingResult.getEntryPoint());
            System.out.println(pendingResult.getFullFieldName());
            System.out.println(pendingResult.getAppliedFault());
            // if
            // (!pendingResult.getAppliedFault().equals(Fault.STRING_OVERFLOW))
            {
                if (!(pendingResult.getOutput() instanceof Exception)) {
                    System.out.println("output Output is: " + Tools.describeObject(pendingResult.getOutput()));
                }
            }

            System.out.println(pendingResult.getOutput().getClass().getName());
        }

        System.out.println("end");
        System.exit(0);
        // domain reduction
        for (String className : classNameAndproblematicFieldsMap.keySet()) {
            String[] xsd = classNameToXsdFileMap.get(className);

            ComplexType declaration = Main.loadComplexType(xsd[0]);
            ComplexType definition = Main.loadComplexType(xsd[1]);
            Main.flatStructure = Helper.buildFlatStructureMap(definition);

            Iterator<Dependency> it = declaration.getAnnotation().getAppinfo().getDependencies().iterator();
            while (it.hasNext()) {
                Dependency dependency = it.next();
                boolean found = false;

                List<String> probFieldsForThisClass = classNameAndproblematicFieldsMap.get(className);

                for (Param param : dependency.getFunction().getParamList()) {
                    String parameter = param.getName();

                    if (param.getName().startsWith("r")) {
                        parameter = Main.flatStructure.get(param.getName()).getElement().getName();
                    }

                    System.out.println("----------->" + probFieldsForThisClass + " param name: " + parameter);

                    if (probFieldsForThisClass.contains(parameter)) {
                        System.out.println("Found problematic field (" + parameter + " from (" + param.getName() + ") for class " + className + " in dependency ID: " + dependency.getId());
                        found = true;
                    }
                }

                if (!found) {
                    it.remove();
                }
            }

            System.out.println("******************* RESULTING XML (declaration) **************************");
            Main.printXml(declaration);
            System.out.println("------------------------------------------------------------");
            Main.printXml(definition);
            System.out.println("******************* **************************");
        }
    }

    public static void main(String[] args) {
        try {
            new Initializer().init(Configuration.dir + "\\robustness-tests");
        } catch (PersistenceException e) {
            e.printStackTrace();
        }
    }
}
