package extra.dao;

import java.sql.SQLException;

import org.springframework.jdbc.core.namedparam.BeanPropertySqlParameterSource;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.jdbc.core.namedparam.SqlParameterSource;
import org.springframework.jdbc.core.support.JdbcDaoSupport;

import extra.pojo.Address;
import extra.pojo.Author;
import extra.pojo.Country;
import extra.pojo.Customer;
import extra.pojo.Item;
import extra.pojo.Order;
import extra.pojo.OrderLine;
import extra.pojo.Stock;

public class InsertionDao extends JdbcDaoSupport
{
	private NamedParameterJdbcTemplate namedParameterJdbcTemplate;

	public void insertAddress(Address address) throws SQLException
	{
		final String sql = "INSERT INTO ADDRESS (ADDR_ID, ADDR_STREET1, ADDR_STREET2, ADDR_CITY, ADDR_STATE, ADDR_ZIP, ADDR_CO_ID)"
		        + "VALUES (:id, :street1, :street2, :city, :state, :zip, :coId)";
		SqlParameterSource paramSource = new BeanPropertySqlParameterSource(
		        address);

		this.namedParameterJdbcTemplate.update(sql, paramSource);
	}

	public void insertAuthor(Author author) throws SQLException
	{
		final String sql = "INSERT INTO AUTHOR (A_ID, A_FNAME, A_LNAME)"
		        + "VALUES (:id, :FName, :LName)";
		SqlParameterSource paramSource = new BeanPropertySqlParameterSource(
		        author);

		this.namedParameterJdbcTemplate.update(sql, paramSource);
	}

	public void insertCountry(Country country) throws SQLException
	{
		final String sql = "INSERT INTO COUNTRY (CO_ID, CO_NAME, CO_TAX, CO_SHIPPING_ZONE)"
		        + "VALUES (:id, :name, :tax, :shippingZone)";
		SqlParameterSource paramSource = new BeanPropertySqlParameterSource(
		        country);

		this.namedParameterJdbcTemplate.update(sql, paramSource);
	}

	public void insertCustomer(Customer costumer) throws SQLException
	{
		final String sql = "INSERT INTO CUSTOMER (C_ID, C_BUSINESS_NAME, C_BUSINESS_INFO, C_PASSWD, C_CONTACT_FNAME, C_CONTACT_LNAME, C_ADDR_ID, C_CONTACT_PHONE, C_CONTACT_EMAIL, C_PAYMENT_METHOD, C_CREDIT_INFO, C_PO, C_DISCOUNT)"
		        + "VALUES (:id, :businessName, :businessInfo, :passwd, :contactFName, :contactLName, :addrId, :contactPhone, :contactEmail, :paymentMethod, :creditInfo, :po, :discount)";
		SqlParameterSource paramSource = new BeanPropertySqlParameterSource(
		        costumer);

		this.namedParameterJdbcTemplate.update(sql, paramSource);
	}

	public void insertItem(Item item) throws SQLException
	{
		final String sql = "INSERT INTO ITEM (I_ID, I_TITLE, I_A_ID, I_PUB_DATE, I_PUBLISHER, I_SUBJECT, I_DESC, I_SRP, I_COST, I_AVAIL, I_ISBN, I_PAGE, I_BACKING, I_DIMENSIONS, I_IMAGE)"
		        + "VALUES (:id, :title, :AId, :pubDate, :publisher, :subject, :desc, :srp, :cost, :avail, :isbn, :page, :backing, :dimensions, :image)";
		SqlParameterSource paramSource = new BeanPropertySqlParameterSource(
		        item);

		this.namedParameterJdbcTemplate.update(sql, paramSource);
	}

	public void insertStock(Stock stock) throws SQLException
	{
		final String sql = "INSERT INTO STOCK (S_I_ID, S_QTY)"
		        + "VALUES (:IId, :qty)";
		SqlParameterSource paramSource = new BeanPropertySqlParameterSource(
		        stock);

		this.namedParameterJdbcTemplate.update(sql, paramSource);
	}

	public void insertOrder(Order order) throws SQLException
	{
		final String sql = "INSERT INTO ORDERS (O_ID, O_C_ID, O_DATE, O_SUB_TOTAL, O_TAX, O_TOTAL, O_PROCESS_DATE, O_SHIP_TYPE, O_SHIP_ADDR_ID, O_STATUS, O_AUTH_ID, O_SHIP_COST, O_DISCOUNT)"
		        + "VALUES (:id, :CId, :date, :subTotal, :tax, :total, :processDate, :shipType, :shipAddrId, :status, :authId, :shipCost, :discount)";
		SqlParameterSource paramSource = new BeanPropertySqlParameterSource(
		        order);

		this.namedParameterJdbcTemplate.update(sql, paramSource);
	}

	public void insertOrderLine(OrderLine orderLine) throws SQLException
	{
		final String sql = "INSERT INTO ORDER_LINE (OL_ID, OL_O_ID, OL_I_ID, OL_QTY, OL_STATUS, OL_I_COST)"
		        + "VALUES (:id, :OId, :IId, :qty, :status, :ICost)";
		SqlParameterSource paramSource = new BeanPropertySqlParameterSource(
		        orderLine);

		this.namedParameterJdbcTemplate.update(sql, paramSource);
	}

	public void setNamedParameterJdbcTemplate(
	        NamedParameterJdbcTemplate namedParameterJdbcTemplate)
	{
		this.namedParameterJdbcTemplate = namedParameterJdbcTemplate;
	}
}
