package pt.uc.dei.wsdbench.sut.dao.rowmapper;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.sql.ResultSet;
import java.sql.SQLException;

import org.springframework.jdbc.core.simple.ParameterizedRowMapper;

import pt.uc.dei.wsdbench.generic.component.tpcapp.Constants;
import pt.uc.dei.wsdbench.sut.exception.SutException;
import pt.uc.dei.wsdbench.sut.service.output.ProductDetailPojo;

public class ProductDetailRowmapper implements ParameterizedRowMapper<ProductDetailPojo>
{

    public ProductDetailPojo mapRow(ResultSet rs, int arg1) throws SQLException
    {
	ProductDetailPojo result = new ProductDetailPojo();

	result.setItemId(rs.getLong("I_ID"));
	result.setItemTitle(rs.getString("I_TITLE"));
	result.setAuthorFName(rs.getString("A_FNAME"));
	result.setAuthorLName(rs.getString("A_LNAME"));
	result.setItemPubDate(rs.getDate("I_PUB_DATE"));
	result.setItemPublisher(rs.getString("I_PUBLISHER"));
	result.setItemSubject(rs.getString("I_SUBJECT"));
	result.setItemDesc(rs.getString("I_DESC"));
	result.setItemCost(rs.getString("I_COST"));
	result.setItemSrp(rs.getString("I_SRP"));
	result.setItemAvail(rs.getString("I_AVAIL"));
	result.setItemIsbn(rs.getString("I_ISBN"));
	result.setItemPage(rs.getString("I_PAGE"));
	result.setItemBacking(rs.getString("I_BACKING"));
	result.setItemDimensions(rs.getString("I_DIMENSIONS"));

	try
	{
	    result.setImageBytes(loadImage(rs.getString("I_IMAGE")));
	} catch (SutException e)
	{
	    throw new SQLException(e.getMessage());
	}

	return result;
    }

    public byte[] getBytesFromFile(String fileName) throws SutException
    {
	File file = new File(fileName);
	InputStream inputStream;
	try
	{
	    inputStream = new FileInputStream(file);
	} catch (Exception e)
	{
	    throw new SutException(e);
	}

	long length = file.length();

	if (length > Integer.MAX_VALUE)
	{
	    throw new SutException("File " + file.getName() + " has invalid length " + file.length()
		    + " for this type of reading.");
	}

	byte[] bytes = new byte[Constants.IMAGE_MAX_SIZE];

	try
	{
	    while (inputStream.read(bytes) != -1)
	    {
		;
	    }
	} catch (IOException e)
	{
	    throw new SutException(e);
	} finally
	{
	    try
	    {
		inputStream.close();
	    } catch (IOException e1)
	    {
		throw new SutException(e1);
	    }
	}

	return bytes;
    }

    private byte[] loadImage(String location) throws SutException
    {
	byte[] result;
	// Base64 base64 = new Base64();
	result = /* base64.encode( */getBytesFromFile(location)/* ) */;

	return result;
    }
}
