package pt.uc.dei.wsdbench.sut.dao.tpcapp;

import java.sql.SQLException;
import java.util.List;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.jdbc.core.namedparam.BeanPropertySqlParameterSource;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.jdbc.core.namedparam.SqlParameterSource;
import org.springframework.jdbc.core.support.JdbcDaoSupport;

import extra.pojo.Address;
import pt.uc.dei.wsdbench.sut.dao.rowmapper.LongRowmapper;

public class NewCustomerDao extends JdbcDaoSupport
{
    private Log log = LogFactory.getLog(NewCustomerDao.class);

    private NamedParameterJdbcTemplate namedParameterJdbcTemplate;

    /**
     * 
     * @param countryName
     * @return CountryId if found, else returns null.
     */
    public Long findCountryIdByName(String countryName) throws SQLException
    {
	Long result = null;
	List list = null;
	String sql = "SELECT CO_ID FROM COUNTRY WHERE CO_NAME=?";

	list = this.getJdbcTemplate().query(sql, new String[]
	{ countryName }, new LongRowmapper());

	if (list != null && list.size() > 0)
	{
	    result = (Long) list.get(0);
	}
	return result;
    }

    /**
     * Searches for a matching address in db.
     * 
     * @param address
     * @return the existent ADDR_ID or null if no matching address was found
     */
    public Long findAddressId(Address address) throws SQLException
    {
	Long result = null;
	List matches = null;

	final String sql = "SELECT ADDR_ID FROM ADDRESS" + " WHERE ADDR_STREET1 = :street1"
		+ " AND ADDR_STREET2 = :street2" + " AND ADDR_CITY = :city" + " AND ADDR_STATE = :state"
		+ " AND ADDR_ZIP = :zip" + " AND ADDR_CO_ID = :coId";

	SqlParameterSource source = new BeanPropertySqlParameterSource(address);

	try
	{
	    matches = this.namedParameterJdbcTemplate.query(sql, source, new LongRowmapper());
	} catch (Exception e)
	{
	    log.error(e);
	}

	if (matches != null && matches.size() > 0)
	{
	    result = (Long) matches.get(0);
	}

	return result;
    }

    public void setNamedParameterJdbcTemplate(NamedParameterJdbcTemplate namedParameterJdbcTemplate)
    {
	this.namedParameterJdbcTemplate = namedParameterJdbcTemplate;
    }

}
