package pt.uc.dei.wsdbench.sut.domain.schema.helper;

import java.util.List;

import pt.uc.dei.wsdbench.sut.domain.schema.exception.ValidationException;
import pt.uc.dei.wsdbench.sut.domain.schema.restriction.Restriction;
import pt.uc.dei.wsdbench.sut.domain.schema.robustness.Function;

public class FunctionExecutor
{

    public static boolean executePartialValidation(List<FunctionValuesAndRestrictions> functionValuesAndRestrictions)
	    throws ValidationException
    {
	boolean result = false;

	for (FunctionValuesAndRestrictions tmp : functionValuesAndRestrictions)
	{
	    result = execute(tmp.getFunction(), tmp.getValueList(), tmp.getRestrictionList());
	    if (!result)
	    {
		throw new ValidationException("Error while executing a partial validation. Function is: "
			+ tmp.getFunction());
	    }
	}

	return result;
    }

    public static boolean execute(Function function, List<Object> argumentValues, List<Restriction> restrictionList)
    {
	boolean result = false;

	if (function.getName().equals("equals") && function.getStrategy().equals("and"))
	{
	    result = true;
	    String argRef = String.valueOf(argumentValues.get(0));

	    for (int i = 1; i < argumentValues.size(); i++)
	    {
		String toCompare = String.valueOf(argumentValues.get(i));
		if (!argRef.equals(toCompare))
		{
		    System.out.println("Equals is not valid. Argument with value (" + toCompare
			    + ") is not equal to reference argument with value (" + argRef + ")");
		    result = false;
		    break;
		}
	    }
	}
	else if (function.getName().equals("equals") && function.getStrategy().equals("or"))
	{
	    result = false;
	    String argRef = String.valueOf(argumentValues.get(0));

	    for (int i = 1; i < argumentValues.size(); i++)
	    {
		String toCompare = String.valueOf(argumentValues.get(i));
		if (argRef.equals(toCompare))
		{
		    result = true;
		    break;
		}
	    }

	    if (result == false)
	    {
		System.out.println("Equals with strategy 'or' did not validate arguments: " + argumentValues);
	    }
	}

	else if (function.getName().equals("aggregator") && function.getStrategy().equals("and"))
	{
	    Object argRef = argumentValues.get(0);
	    // TODO null is not treated yet IMPLEMENT THIS!!!!!!
	    // VERIFICAR SE NULL FAZ PARTE DO DOMINIO
	    // NESTE CASO ESTOU A ASSUMIR QUE NULL  INVLIDO, DEVIA TER SIDO
	    // LIDO DO XSD NILLABLE OU MIN OCCURS
	    if (argumentValues.get(0) == null)
	    {
		System.out.println("null argument found. function not applied!");
		return false;
	    }
	    
	    Restriction restriction = restrictionList.get(0);

	    result = FunctionHelper.fitsRestriction(restriction, argRef);
	    checkResult(restriction, argRef, result);

	    for (int i = 1; i < argumentValues.size(); i++)
	    {
		result = FunctionHelper.fitsRestriction(restrictionList.get(i), argumentValues.get(i));
		checkResult(restrictionList.get(i), argumentValues.get(i), result);
	    }
	}
	else
	{
	    result = false;
	    System.out.println("Function (name = " + function.getName() + " strategy = " + function.getStrategy()
		    + ") not implemented!");
	}

	if (result == true)
	{
	    // System.out.println("Function (name = " + function.getName() +
	    // " strategy = " + function.getStrategy()
	    // + ") applied successfully for argument values: " +
	    // argumentValues);
	}

	return result;
    }

    private static void checkResult(Restriction restriction, Object argRef, boolean result)
    {
	if (!result)
	{
	    System.out.println("Argument (" + argRef + ") with restriction id (" + restriction.getId()
		    + ") did not fit the restriction");
	}
    }
}
