package pt.uc.dei.wsdbench.sut.domain.schema.helper;

import java.lang.reflect.Field;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.springframework.util.StringUtils;

import pt.uc.dei.wsdbench.sut.domain.schema.Main;
import pt.uc.dei.wsdbench.sut.domain.schema.exception.ValidationException;
import pt.uc.dei.wsdbench.sut.domain.schema.restriction.Restriction;
import pt.uc.dei.wsdbench.sut.domain.schema.robustness.Dependency;
import pt.uc.dei.wsdbench.sut.domain.schema.robustness.Function;
import pt.uc.dei.wsdbench.sut.domain.schema.robustness.Param;
import pt.uc.dei.wsdbench.sut.domain.schema.type.ComplexType;
import pt.uc.dei.wsdbench.sut.domain.schema.type.Element;
import pt.uc.dei.wsdbench.sut.injector.tools.Tools;
import pt.uc.dei.wsdbench.sut.service.output.NewProductsOutput;

public class Helper
{
    public static Map<String, ComplexTypeAndElementAndRestriction> buildFlatStructureMap(
	    ComplexType complexTypeDefinition)
    {
	Map<String, ComplexTypeAndElementAndRestriction> result = new HashMap<String, ComplexTypeAndElementAndRestriction>();

	for (Element element : complexTypeDefinition.getSequence())
	{
	    for (Restriction restriction : element.getSimpleType().getRestrictionList())
	    {
		ComplexTypeAndElementAndRestriction value = new ComplexTypeAndElementAndRestriction(
			complexTypeDefinition, element, restriction);
		result.put(restriction.getId(), value);
	    }
	}

	return result;
    }

    @Deprecated
    public static Element findElementByParam(Param param, ComplexType complexType)
    {
	Element result = null;

	for (Element el : complexType.getSequence())
	{
	    if (param.getName().equals(el.getName()))
	    {
		result = el;
		break;
	    }
	}

	if (result == null)
	{
	    throw new RuntimeException("Unable to find element with name(" + param.getName()
		    + "). Should never happen.");
	}

	return result;
    }

    /**
     * Procura o valor do atributo de nome name no argumento de indice index.
     * Assume-se que o atributo existe.
     * 
     * @param args
     * @param index
     * @param name
     * @return
     */
    public static Object findValueByIndexAndParameterName(Object[] args, Integer index, String name)
    {
	Object result = null;

	try
	{
	    Object obj = args[index];
	    Field field = obj.getClass().getDeclaredField(name);
	    field.setAccessible(true);
	    result = field.get(obj);
	}
	catch (SecurityException e)
	{
	    // TODO Auto-generated catch block
	    e.printStackTrace();
	}
	catch (IllegalArgumentException e)
	{
	    // TODO Auto-generated catch block
	    e.printStackTrace();
	}
	catch (NoSuchFieldException e)
	{
	    // empty
	}
	catch (IllegalAccessException e)
	{
	    // TODO Auto-generated catch block
	    e.printStackTrace();
	}

	return result;
    }

    static String dir = "D:\\svn\\phd\\apps\\sut-native-dsn-2009\\";

    public static boolean finalValidation(Object[] arguments) throws ValidationException
    {
	//TODO uncomment
	//System.out.println("----Applying final validation to " + Tools.describeObject(arguments[0]));
	
	if (arguments[0] instanceof NewProductsOutput)
	{
	    if ( (((NewProductsOutput)arguments[0]).getNewProductsList() != null))
	    {
		return true;
	    }
	}
	
	boolean result;

	if (arguments[0] instanceof Exception)
	{
	    throw new ValidationException((Exception) arguments[0]);
	}

	String filename = StringUtils.unqualify(arguments[0].getClass().getName());
	ComplexType decl = Main.loadComplexType(dir + filename + "Declaration.xsd");
	ComplexType def = Main.loadComplexType(dir + filename + "Definition.xsd");

	result = Helper.finalValidateArguments(arguments, decl, def);

	return result;
    }

    private static boolean finalValidateArguments(Object[] arguments, ComplexType complexTypeDeclaration,
	    ComplexType complexTypeDefinition) throws ValidationException
    {
	boolean result = false;
	Main.flatStructure = buildFlatStructureMap(complexTypeDefinition);
	List<Dependency> dependencies = complexTypeDeclaration.getAnnotation().getAppinfo().getDependencies();

	for (Dependency dependency : dependencies)
	{
	    List<Object> valueList = new ArrayList<Object>();
	    List<Restriction> restrictionList = new ArrayList<Restriction>();

	    Function function = dependency.getFunction();

	    for (Param param : function.getParamList())
	    {
		Object value;
		String paramName = param.getName();

		if (function.getName().equals("aggregator"))
		{
		    ComplexTypeAndElementAndRestriction tmp = Main.flatStructure.get(param.getName());
		    if (tmp == null)
		    {
			throw new RuntimeException("Function argument not found. param.getName() = " + param.getName());
		    }
		    paramName = tmp.getElement().getName();
		    restrictionList.add(tmp.getRestriction());
		}

		value = findValueByIndexAndParameterName(arguments, param.getIndex(), paramName);
		valueList.add(value);
	    }

	    result = FunctionExecutor.execute(function, valueList, restrictionList);

	    if (result == false)
	    {
		throw new ValidationException("FUNCTION DOES NOT VALIDATE!: " + Tools.describeObject(function) + " "
			+ valueList + " " + Tools.describeObject(restrictionList));
	    }
	}

	return result;
    }

    public static boolean validateArguments(Object[] arguments, ComplexType complexTypeDeclaration,
	    ComplexType complexTypeDefinition)
    {
	boolean result = false;

	List<Dependency> dependencies = complexTypeDeclaration.getAnnotation().getAppinfo().getDependencies();

	for (Dependency dependency : dependencies)
	{
	    List<Object> valueList = new ArrayList<Object>();
	    List<Restriction> restrictionList = new ArrayList<Restriction>();

	    Function function = dependency.getFunction();

	    for (Param param : function.getParamList())
	    {
		Object value;
		String paramName = param.getName();

		if (function.getName().equals("aggregator"))
		{
		    ComplexTypeAndElementAndRestriction tmp = Main.flatStructure.get(param.getName());
		    if (tmp == null)
		    {
			throw new RuntimeException("Function argument not found. param.getName() = " + param.getName());
		    }
		    paramName = tmp.getElement().getName();
		    restrictionList.add(tmp.getRestriction());
		}

		value = findValueByIndexAndParameterName(arguments, param.getIndex(), paramName);
		valueList.add(value);
	    }

	    result = FunctionExecutor.execute(function, valueList, restrictionList);

	    if (result == false)
	    {
		break;
	    }
	}

	return result;
    }
}
