package pt.uc.dei.wsdbench.sut.injector;

import java.util.ArrayList;
import java.util.List;

import org.apache.commons.lang.RandomStringUtils;
import org.apache.commons.lang.math.RandomUtils;

import pt.uc.dei.wsdbench.sut.injector.core.domain.Domain;
import pt.uc.dei.wsdbench.sut.injector.core.domain.StringDomain;

public class FaultInjector
{

    public static Object inject(Domain domain, Object originalValue, String faultType)
    {
	Object result = null;
	
	if (originalValue instanceof Long)
	{
	    result = handleLong(domain, (Long)originalValue, faultType);
	}
	else if (originalValue instanceof String)
	{
	    result = handleString(domain, (String) originalValue, faultType);
	}
	else if (originalValue instanceof List)
	{
	    result = handleList(domain, (List<Long>) originalValue, faultType);
	}	
	else
	{
	    throw new RuntimeException("Unsupported type. (originalValue):" + originalValue + " type is: "
		    + originalValue.getClass().getName() + " faultType: " + faultType);
	}

	return result;
    }

    private static Long handleLong(Domain domain, Long originalValue, String faultType)
    {
	Long result = null;
	Domain Domain = domain;
	
	if (faultType.equals(Fault.NUMBER_ABSOLUTE_MINUS_ONE))
	{
	    result = Long.MIN_VALUE;
	}
	else if (faultType.equals(Fault.NUMBER_ABSOLUTE_ONE))
	{
	    result = 1L;
	}
	else if (faultType.equals(Fault.NUMBER_ABSOLUTE_ZERO))
	{
	    result = 0L;
	}
	else if (faultType.equals(Fault.NUMBER_MAX_DOMAIN))
	{
	    result = Domain.getMaxSize();
	}
	else if (faultType.equals(Fault.NUMBER_MAX_DOMAIN_MINUS_ONE))
	{
	    result = Domain.getMaxSize() - 1;
	}
	else if (faultType.equals(Fault.NUMBER_MAX_DOMAIN_PLUS_ONE))
	{
	    result = Domain.getMaxSize() + 1;
	}
	else if (faultType.equals(Fault.NUMBER_MAX_TYPE))
	{
	    result = Long.MAX_VALUE;
	}
	else if (faultType.equals(Fault.NUMBER_MIN_DOMAIN))
	{
	    result = Domain.getMinSize();
	}
	else if (faultType.equals(Fault.NUMBER_MIN_DOMAIN_MINUS_ONE))
	{
	    result = Domain.getMinSize() - 1;
	}
	else if (faultType.equals(Fault.NUMBER_MIN_DOMAIN_PLUS_ONE))
	{
	    result = Domain.getMinSize() + 1;
	}
	else if (faultType.equals(Fault.NUMBER_MIN_TYPE))
	{
	    result = Long.MIN_VALUE;
	}
	else if (faultType.equals(Fault.NUMBER_MINUS_ONE))
	{
	    result = originalValue - 1;
	}
	else if (faultType.equals(Fault.NUMBER_NULL))
	{
	    result = null;
	}
	else if (faultType.equals(Fault.NUMBER_PLUS_ONE))
	{
	    result = originalValue + 1;
	}

	return result;
    }
    

    private static String handleString(Domain domain, String originalValue, String faultType)
    {
	String result = null;
	StringDomain stringDomain = (StringDomain) domain; 
	
	Integer acceptableLength = (int) (RandomUtils.nextInt((int) (stringDomain.getMaxSize() - stringDomain.getMinSize())) + stringDomain
		.getMinSize());

	if (faultType.equals(Fault.STRING_ADD_NON_PRINTABLE))
	{
	    result = originalValue + "\b\t\r\n";
	}
	else if (faultType.equals(Fault.STRING_ALPHANUMERIC))
	{

	    result = RandomStringUtils.randomAlphanumeric(acceptableLength);
	}
	else if (faultType.equals(Fault.STRING_EMPTY))
	{
	    result = "";
	}
	else if (faultType.equals(Fault.STRING_NON_PRINTABLE))
	{
	    result = "\b\t\r\n";
	}
	else if (faultType.equals(Fault.STRING_NULL))
	{
	    result = null;
	}
	else if (faultType.equals(Fault.STRING_OVERFLOW))
	{
	    result = "sedgsefrgsodifuhgaoihgapwrihuga+wosuvh+AOWUDVHA+SUFDHVPSAUFHVPAUSFHDVPIAUSFHDBVPAUSDHVPOAIUHFDVPIAUSFGVPAISUDVBLAKSJDVBLKjbpibsidkfjbvksdfbvzdfbvouzdsfhpvoiszhdmcpoivhasmdoifhmxpoidshfvmpormheopvaihfpdovhapdofhpoamdfhbpovadmfpvbohampofbimhao+fbsdofsedgsefrgsodifuhgaoihgapwrihuga+wosuvh+AOW";
	}
	else if (faultType.equals(Fault.STRING_PREDEFINED))
	{
	    result = "'";
	}

	return result;
    }

    // TODO aumentar o tipo de falhas para considerar o dominio
    private static List<Long> handleList(Domain domain, List<Long> originalValue, String faultType)
    {
	List<Long> result = null;
	
	if (faultType.equals(Fault.LIST_ADD_ELEMENT))
	{
	    Long toAdd = RandomUtils.nextLong();
	    originalValue.add(toAdd);
	    result = originalValue;
	}
	else if (faultType.equals(Fault.LIST_DUPLICATE_ELEMENTS))
	{
	    if (originalValue.size() > 0)
	    {
		originalValue.add(0, originalValue.get(0));
	    }
	}
	else if (faultType.equals(Fault.LIST_EMPTY))
	{
	    result = new ArrayList<Long>();
	}
	else if (faultType.equals(Fault.LIST_NULL))
	{
	    result = null;
	}
	else if (faultType.equals(Fault.LIST_REDUCE_TO_ONE))
	{
	    int index = RandomUtils.nextInt(originalValue.size());
	    result = new ArrayList<Long>();
	    result.add(originalValue.get(index));
	}
	else if (faultType.equals(Fault.LIST_REMOVE_ELEMENT))
	{
    	    if (originalValue.size() > 0)
    	    {
    		int index = RandomUtils.nextInt(originalValue.size());
    		originalValue.remove(index);
    	    }
    	    result = originalValue;
	}

	return result;
    }
}