package pt.uc.dei.wsdbench.sut.injector.core;

import java.io.IOException;
import java.lang.reflect.Field;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Hashtable;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.aspectj.lang.ProceedingJoinPoint;
import org.springframework.util.StringUtils;

import pt.uc.dei.wsdbench.sut.injector.Fault;
import pt.uc.dei.wsdbench.sut.injector.FaultInjector;
import pt.uc.dei.wsdbench.sut.injector.PersistenceException;
import pt.uc.dei.wsdbench.sut.injector.configuration.Configuration;
import pt.uc.dei.wsdbench.sut.injector.core.domain.Domain;
import pt.uc.dei.wsdbench.sut.injector.persistence.PersistenceManager;
import pt.uc.dei.wsdbench.sut.injector.result.PendingResult;
import pt.uc.dei.wsdbench.sut.injector.tools.Tools;

public class Core
{
    private static PersistenceManager<PendingResult> persistenceTools = new PersistenceManager<PendingResult>();

    private static PendingResult pendingResult;

    private static List<PendingResult> history = new ArrayList<PendingResult>();

    private static Configuration configuration = new Configuration();

    // sourceLocation, <field,faultList>
    private static Map<String, Map<String, List<String>>> mainMap = new Hashtable<String, Map<String, List<String>>>();

    // TODO estratgia para escolha do campo. Aleatria, ordenada pela chave,
    // ordenada pelo nmero de falhas que faltam.. outra? devia ser pelo numero
    // de invocaes
    private static Map<String, List<String>> buildFaultListForAllObjectFields(String sourceLocation, Object object)
	    throws IllegalArgumentException, IllegalAccessException
    {
	Map<String, List<String>> result = new HashMap<String, List<String>>();

	Field[] fields = object.getClass().getDeclaredFields();

	for (int i = 0; i < fields.length; i++)
	{
	    fields[i].setAccessible(true);
	    System.out.println("About to create fault list for field: " + fields[i].getName());
	    List<String> faultList = Fault.createFaultList(fields[i].get(object));
	    String fieldName = fields[i].getDeclaringClass().getCanonicalName() + "." + fields[i].getName();
	    result.put(fieldName, faultList);
	}

	return result;
    }

    // TODO assuming only one parameter
    public static void tryAddingFaultsToMainMap(String sourceLocation, Object parameter)
	    throws IllegalArgumentException, IllegalAccessException
    {
	// fieldName, faultString
	Map<String, List<String>> completeFaultMap = mainMap.get(sourceLocation);

	if (completeFaultMap == null)
	{
	    completeFaultMap = buildFaultListForAllObjectFields(sourceLocation, parameter);
	    mainMap.put(sourceLocation, completeFaultMap);
	}
    }

    public static FullFieldNameAndFaultType findParameterNameAndFaultType(String sourceLocation)
    {
	FullFieldNameAndFaultType result = null;

	// field faultList
	Map<String, List<String>> completeFaultMap = mainMap.get(sourceLocation);

	Set<String> fields = completeFaultMap.keySet();

	for (String field : fields)
	{
	    List<String> faultList = completeFaultMap.get(field);
	    if (faultList.size() > 0)
	    {
		result = new FullFieldNameAndFaultType(field, faultList.remove(0));
		break;
	    }
	}

	return result;
    }

    public static Object applyFault(String sourceCodeLine, FullFieldNameAndFaultType nameAndFault, Object[] args)
	    throws IllegalArgumentException, SecurityException, IllegalAccessException, NoSuchFieldException
    {
	Object result;

	System.out.println("Applying Fault: " + nameAndFault.getFaultType() + " to parameter: "
		+ nameAndFault.getName());

	Object originalValue = findAttributeValueByFullFieldName(args, nameAndFault.getName());
	Domain domain = Core.pendingResult.findAndSetDomain(sourceCodeLine, nameAndFault.getName(), originalValue);
	Core.pendingResult.setOriginalValue(originalValue);

	result = FaultInjector.inject(domain, originalValue, nameAndFault.getFaultType());

	Core.pendingResult.setMutatedFieldValue(result);
	Core.pendingResult.setAppliedFault(nameAndFault.getFaultType());
	Core.pendingResult.setFullFieldName(nameAndFault.getName());
	Core.pendingResult.setOutput(result);

	return result;
    }

    public static Object findAttributeValueByFullFieldName(Object[] args, String fullFieldName)
	    throws IllegalArgumentException, SecurityException, IllegalAccessException, NoSuchFieldException
    {
	Object result;

	result = findOrSetValueOnOriginalArgument(args, fullFieldName, null, false);

	return result;
    }

    public static Object setNewValueOnOriginalArgument(Object[] args, String fullFieldName, Object newValue)
	    throws IllegalArgumentException, SecurityException, IllegalAccessException, NoSuchFieldException
    {
	Object result;

	result = findOrSetValueOnOriginalArgument(args, fullFieldName, newValue, true);

	return result;
    }

    private static Object findOrSetValueOnOriginalArgument(Object[] args, String fullFieldName, Object newValue,
	    boolean setField) throws IllegalArgumentException, IllegalAccessException, SecurityException,
	    NoSuchFieldException
    {
	Object result = null;

	int endIndex = fullFieldName.lastIndexOf(".");
	boolean done = false;

	if (endIndex == -1)
	{
	    throw new RuntimeException("Unable to find '.' in full attribute name.");
	}

	String className = fullFieldName.substring(0, endIndex);
	String targetFieldName = StringUtils.unqualify(fullFieldName);
	for (Object arg : args)
	{
	    if (arg.getClass().getCanonicalName().equals(className))
	    {
		Field field = arg.getClass().getDeclaredField(targetFieldName);
		field.setAccessible(true);

		if (setField)
		{
		    field.set(arg, newValue);
		}
		else
		{
		    result = field.get(arg);
		}
		done = true;
		break;
	    }
	}

	if (!done)
	{
	    throw new RuntimeException("Unable to set or get field. Parameters: fullFieldName = " + fullFieldName
		    + " newValue = " + newValue + " setField = " + setField);
	}

	return result;
    }

    public static void beforeMainService(ProceedingJoinPoint proceedingJoinPoint) throws IllegalArgumentException,
	    IllegalAccessException, SecurityException, NoSuchFieldException, IOException
    {
	Core.pendingResult = new PendingResult();
	Core.pendingResult.setEntryPoint(Tools.retrieveSourceLocation(proceedingJoinPoint));
	Core.pendingResult.setStartTime();

	Object copy = Tools.deepCopy(proceedingJoinPoint.getArgs()[0]);
	Core.pendingResult.setInput(copy);

	Core.tryAddingFaultsToMainMap(Tools.retrieveSourceLocation(proceedingJoinPoint),
		proceedingJoinPoint.getArgs()[0]);
	FullFieldNameAndFaultType nameAndFault = Core.findParameterNameAndFaultType(Tools
		.retrieveSourceLocation(proceedingJoinPoint));

	// tenho um campo e falha para aplicar
	if (nameAndFault != null)
	{
	    Core.pendingResult.setFaultLocation(Tools.retrieveSourceLocation(proceedingJoinPoint));
	    Object mutatedValue = Core.applyFault(Tools.retrieveSourceLocation(proceedingJoinPoint), nameAndFault,
		    proceedingJoinPoint.getArgs());

	    Core.setNewValueOnOriginalArgument(proceedingJoinPoint.getArgs(), nameAndFault.getName(), mutatedValue);
	}
    }

    public static void afterMainService(Object result, ProceedingJoinPoint proceedingJoinPoint)
    {
	Core.pendingResult.setEndTime();
	Core.pendingResult.setOutput(result);
    }

    public static void afterNestedInvocation(Object result, ProceedingJoinPoint proceedingJoinPoint)
	    throws IllegalArgumentException, IllegalAccessException, SecurityException, NoSuchFieldException,
	    IOException
    {
	if (!Core.pendingResult.isDirty())
	{
	    String sourceLocation = Tools.retrieveSourceLocation(proceedingJoinPoint);

	    Core.tryAddingFaultsToMainMap(sourceLocation, result);
	    FullFieldNameAndFaultType nameAndFault = Core.findParameterNameAndFaultType(Tools
		    .retrieveSourceLocation(proceedingJoinPoint));

	    // tenho um campo e falha para aplicar

	    if (nameAndFault != null && ((nameAndFault.getFaultType() == null) || nameAndFault.getName() == null))
	    {
		System.out.println("ERROR IN NAME_AND_FAULT!!!!!!!!!");
		System.out.println(Tools.describeObject(nameAndFault));
		throw new RuntimeException();
	    }

	    if (nameAndFault != null)
	    {
		Object resultCopy = Tools.deepCopy(result);
		Core.pendingResult.setInput(resultCopy);

		Core.pendingResult.setFaultLocation(Tools.retrieveSourceLocation(proceedingJoinPoint));
		Object mutatedValue = Core.applyFault(sourceLocation, nameAndFault, new Object[]
		{ result });

		Core.setNewValueOnOriginalArgument(new Object[]
		{ result }, nameAndFault.getName(), mutatedValue);
	    }
	}
    }

    public static PendingResult getPendingResult()
    {
	return pendingResult;
    }

    public static void setPendingResult(PendingResult pendingResult)
    {
	Core.pendingResult = pendingResult;
    }

    public static Configuration getConfiguration()
    {
	return configuration;
    }

    public static void saveHistory() throws PersistenceException
    {
	persistenceTools.save(Core.pendingResult);
    }

}
