package pt.uc.dei.wsdbench.sut.injector.result;

import java.util.Calendar;

import pt.uc.dei.wsdbench.sut.injector.core.Core;
import pt.uc.dei.wsdbench.sut.injector.core.domain.Domain;
import pt.uc.dei.wsdbench.sut.injector.tools.Tools;

public class PendingResult
{
    private String entryPoint;

    private String faultLocation;

    private Calendar startTime;

    private Calendar endTime;

    private Long durationInMillis;

    private String fullFieldName;

    private Domain domain;

    private String appliedFault;

    private Object originalFieldValue;

    private Object mutatedFieldValue;

    private Object input;

    private Object output;

    public PendingResult()
    {
    }

    @Override
    public String toString()
    {
	String result;
	String domainMin = null;
	String domainMax = null;

	if (domain != null)
	{
	    domainMin = String.valueOf(((Domain) domain).getMinSize());
	    domainMax = String.valueOf(((Domain) domain).getMaxSize());
	}

	result = entryPoint + ", " + faultLocation + ", " + durationInMillis + ", ";
	result += fullFieldName + ", " + domainMin + ", " + domainMax + ", ";
	result += originalFieldValue + ", ";
	result += appliedFault + ", ";
	result += mutatedFieldValue;
	result += ", " + Tools.describeObject(input);

	if (output instanceof Exception)
	{
	    result += ", " + ((Exception) output).getClass().getName();
	}
	else
	{
	    result += ", " + Tools.describeObject(output);
	}

	return result;
    }

    public Calendar getStartTime()
    {
	return startTime;
    }

    public void setStartTime()
    {
	this.startTime = Calendar.getInstance();
    }

    public Calendar getEndTime()
    {
	return endTime;
    }

    public void setEndTime()
    {
	this.endTime = Calendar.getInstance();
	this.durationInMillis = this.endTime.getTimeInMillis() - this.startTime.getTimeInMillis();
    }

    public Long getDurationInMillis()
    {
	return durationInMillis;
    }

    public void setDurationInMillis(Long durationInMillis)
    {
	this.durationInMillis = durationInMillis;
    }

    public Object getOriginalValue()
    {
	return originalFieldValue;
    }

    public void setOriginalValue(Object originalValue)
    {
	this.originalFieldValue = originalValue;
    }

    public String getAppliedFault()
    {
	return appliedFault;
    }

    public void setAppliedFault(String appliedFault)
    {
	this.appliedFault = appliedFault;
    }

    public String getFullFieldName()
    {
	return fullFieldName;
    }

    public void setFullFieldName(String fullFieldName)
    {
	this.fullFieldName = fullFieldName;
    }

    public Object getOriginalFieldValue()
    {
	return originalFieldValue;
    }

    public void setOriginalFieldValue(Object originalFieldValue)
    {
	this.originalFieldValue = originalFieldValue;
    }

    public Object getMutatedFieldValue()
    {
	return mutatedFieldValue;
    }

    public void setMutatedFieldValue(Object mutatedFieldValue)
    {
	this.mutatedFieldValue = mutatedFieldValue;
    }

    public void setStartTime(Calendar startTime)
    {
	this.startTime = startTime;
    }

    public void setEndTime(Calendar endTime)
    {
	this.endTime = endTime;
    }

    public Object getInput()
    {
	return input;
    }

    public void setInput(Object input)
    {
	this.input = input;
    }

    public Object getOutput()
    {
	return output;
    }

    public void setOutput(Object output)
    {
	this.output = output;
    }

    public boolean isDirty()
    {
	boolean result = false;

	result = (this.faultLocation != null);

	return result;
    }

    public Domain findAndSetDomain(String sourceCodeLine, String fullFieldName, Object parameter)
    {
	Domain domain = null;

	domain = Core.getConfiguration().buildDomain(sourceCodeLine + "." + fullFieldName, parameter);

	if ((domain == null) || !domain.isValid())
	{
	    throw new RuntimeException("Unable to fill domain. Parameters are: " + sourceCodeLine + " " + fullFieldName
		    + " " + parameter);
	}

	Core.getPendingResult().setDomain(domain);
	
	return domain;
    }

    public void print()
    {
	System.out.println(this);
    }

 

    public Domain getDomain()
    {
	return domain;
    }

    public void setDomain(Domain domain)
    {
	this.domain = domain;
    }

    public String getEntryPoint()
    {
	return entryPoint;
    }

    public void setEntryPoint(String entryPoint)
    {
	this.entryPoint = entryPoint;
    }

    public String getFaultLocation()
    {
	return faultLocation;
    }

    public void setFaultLocation(String faultLocation)
    {
	this.faultLocation = faultLocation;
    }
}
