package pt.uc.dei.wsdbench.sut.rmi;

import java.rmi.NotBoundException;
import java.rmi.RemoteException;
import java.rmi.registry.LocateRegistry;
import java.rmi.registry.Registry;
import java.rmi.server.UnicastRemoteObject;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import pt.uc.dei.wsdbench.generic.component.rmi.ConfigurationObject;
import pt.uc.dei.wsdbench.generic.component.rmi.IConfigurable;
import pt.uc.dei.wsdbench.generic.component.rmi.INotifiable;
import pt.uc.dei.wsdbench.generic.component.spring.BeanFactory;
import pt.uc.dei.wsdbench.generic.component.tools.Tools;
import pt.uc.dei.wsdbench.generic.component.tpcapp.Constants;
import pt.uc.dei.wsdbench.sut.configuration.Configurator;

public class ConfigurableSut extends UnicastRemoteObject implements IConfigurable {

    Log log = LogFactory.getLog(ConfigurableSut.class);

    public ConfigurableSut() throws RemoteException {
        Registry registry = LocateRegistry.createRegistry(Constants.RMI_PORT_SUT);
        registry.rebind(Constants.RMI_NAME_SUT, this);
    }

    public void configure(ConfigurationObject object) throws RemoteException {
        Configurator configurator = (Configurator) BeanFactory.getBean("configurator");
        Tools.copyFieldValues(configurator, object);
        UnicastRemoteObject.unexportObject((IConfigurable) this, true);
        notifyEnd();
    }

    public void notifyEnd() {
        Configurator configurator = (Configurator) BeanFactory.getBean("configurator");

        try {
            Registry registry = LocateRegistry.getRegistry(configurator.getCoordinatorIp(), configurator.getCoordinatorPort());
            INotifiable notifiable = (INotifiable) registry.lookup(Constants.RMI_NAME_COORDINATOR);
            notifiable.notifyEnd(Constants.RMI_NAME_SUT);
        } catch (RemoteException e) {
            log.error("Exception:", e);
        } catch (NotBoundException e) {
            log.error("Exception:", e);
        }

    }

    public boolean ping() throws RemoteException {
        return true;
    }
}
