package pt.uc.dei.wsdbench.sut.service.delegate.impl.tpcapp;

import java.util.Calendar;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.commons.validator.ValidatorException;
import org.springframework.transaction.TransactionStatus;
import org.springframework.transaction.support.TransactionCallback;
import org.springframework.transaction.support.TransactionTemplate;

import extra.pojo.Address;
import extra.pojo.Customer;
import pt.uc.dei.wsdbench.generic.component.tpcapp.Lcg64RngTools;
import extra.dao.InsertionDao;
import pt.uc.dei.wsdbench.sut.configuration.Configurator;
import pt.uc.dei.wsdbench.sut.dao.tpcapp.NewCustomerDao;
import pt.uc.dei.wsdbench.sut.dbms.core.SequenceHolder;
import pt.uc.dei.wsdbench.sut.exception.SutException;
import pt.uc.dei.wsdbench.sut.external.LongWrapper;
import pt.uc.dei.wsdbench.sut.external.Pov;
import pt.uc.dei.wsdbench.sut.service.delegate.INewCustomerServiceDelegate;
import pt.uc.dei.wsdbench.sut.service.input.NewCustomerInput;
import pt.uc.dei.wsdbench.sut.service.output.NewCustomerOutput;
import pt.uc.dei.wsdbench.sut.service.validator.NewCustomerServiceValidator;

public class NewCustomerServiceDelegate implements INewCustomerServiceDelegate
{
    private Lcg64RngTools lcg64RngTools;

    public Log log = LogFactory.getLog(NewCustomerServiceDelegate.class);

    private NewCustomerDao newCustomerDao;

    private InsertionDao insertionDao;

    private TransactionTemplate transactionTemplate;

    private SequenceHolder sequenceHolder;

    private Pov pov;

    private Exception state;

    private NewCustomerServiceValidator newCustomerServiceValidator;

    private Configurator configurator;

    public NewCustomerServiceDelegate()
    {
	this.lcg64RngTools = new Lcg64RngTools(Calendar.getInstance().getTimeInMillis());
    }

    public NewCustomerOutput process(final NewCustomerInput input) throws Exception
    {
	NewCustomerOutput result = new NewCustomerOutput();

	if (this.configurator.isUseValidation())
	{
	    try
	    {
		this.newCustomerServiceValidator.validate(input);
	    }
	    catch (ValidatorException e)
	    {
		throw new SutException(e);
	    }
	}

	if (input.getPaymentMethod().equals("PO"))
	{
	    LongWrapper wrapper = this.pov.call(null, null);

	    wrapper.getValue().getClass();
	    // TODO INVOKE REAL POV EXTERNAL SYSTEM
	}

	result = (NewCustomerOutput) this.transactionTemplate.execute(new TransactionCallback()
	{
	    public Object doInTransaction(TransactionStatus status)
	    {
		NewCustomerOutput output = null;
		try
		{
		    Long countryId = newCustomerDao.findCountryIdByName(input.getBillingCountry());
		    if (countryId != null)
		    {
			Address address = buildAddress(countryId, input);
			Long addressId = newCustomerDao.findAddressId(address);
			boolean insert = false;
			if (addressId == null)
			{
			    addressId = sequenceHolder.nextAddress();
			    insert = true;
			}

			address.setId(addressId);
			if (insert)
			{
			    insertionDao.insertAddress(address);
			}
			Long customerId = sequenceHolder.nextCustomer();
			Customer customer = buildCustomer(input, customerId, address.getId());
			customer.setDiscount(generateDiscount());
			insertionDao.insertCustomer(customer);
			output = buildOutput(customerId);
		    }
		    else
		    {
			throw new IllegalArgumentException("Country (" + input.getBillingCountry()
				+ ") not found in database");
		    }

		}
		catch (Exception e)
		{
		    log.error(e);
		    status.setRollbackOnly();
		    state = e;
		}

		return output;
	    }
	});

	if (state != null)
	{
	    throw state;
	}

	return result;
    }

    public NewCustomerOutput buildOutput(Long customerId)
    {
	NewCustomerOutput result = new NewCustomerOutput();

	result.setCustomerId(customerId);
	// result.setSystemIdentifier((String)
	// BeanFactory.getBean("systemIdentifier"));

	return result;
    }

    public Double generateDiscount()
    {
	Double result;

	result = Double.valueOf((lcg64RngTools.Next(0, 5000) / 100));

	return result;
    }

    public Customer buildCustomer(NewCustomerInput inputObj, long customerId, long addressId)
    {
	Customer result = new Customer();
	result.setAddrId(addressId);
	result.setBusinessInfo(inputObj.getBusinessInfo());
	result.setBusinessName(inputObj.getBusinessName());
	result.setContactEmail(inputObj.getContactEmail());
	result.setContactFName(inputObj.getContactFName());
	result.setContactLName(inputObj.getContactLName());
	result.setContactPhone(inputObj.getContactPhone());
	result.setCreditInfo(inputObj.getCreditInfo());
	result.setId(customerId);
	result.setPasswd(inputObj.getPassword());
	result.setPaymentMethod(inputObj.getPaymentMethod());
	result.setPo(inputObj.getPoId());

	return result;
    }

    public Address buildAddress(Long coId, NewCustomerInput inputObj)
    {
	Address result = new Address();

	result.setCity(inputObj.getBillingCity());
	result.setCoId(coId);
	result.setState(inputObj.getBillingState());
	result.setStreet1(inputObj.getBillingAddr1());
	result.setStreet2(inputObj.getBillingAddr2());
	result.setZip(inputObj.getBillingZip());

	return result;
    }

    public void setNewCustomerDao(NewCustomerDao newCustomerDao)
    {
	this.newCustomerDao = newCustomerDao;
    }

    public void setSequenceHolder(SequenceHolder sequenceHolder)
    {
	this.sequenceHolder = sequenceHolder;
    }

    public void setTransactionTemplate(TransactionTemplate transactionTemplate)
    {
	this.transactionTemplate = transactionTemplate;
    }

    public void setInsertionDao(InsertionDao insertionDao)
    {
	this.insertionDao = insertionDao;
    }

    public void setPov(Pov pov)
    {
	this.pov = pov;
    }

    public void setNewCustomerServiceValidator(NewCustomerServiceValidator newCustomerServiceValidator)
    {
	this.newCustomerServiceValidator = newCustomerServiceValidator;
    }

    public void setConfigurator(Configurator configurator)
    {
	this.configurator = configurator;
    }

}
