package pt.uc.dei.wsdbench.sut.service.validator;

import java.lang.reflect.Field;
import java.util.List;

import org.apache.commons.logging.Log;
import org.apache.commons.validator.EmailValidator;
import org.apache.commons.validator.ValidatorException;

import pt.uc.dei.wsdbench.generic.component.pojo.Country;
import pt.uc.dei.wsdbench.loader.configuration.CountryReader;
import pt.uc.dei.wsdbench.sut.service.input.NewCustomerInput;

public class GenericValidator {
	private Log log;

	private CountryReader countryReader;

	public void validateFields(Object obj) throws ValidatorException {
		if (obj == null) {
			throw new ValidatorException("Null object supplied!");
		}

		Field[] fields = obj.getClass().getDeclaredFields();

		for (Field field : fields) {
			field.setAccessible(true);
			try {
				Object value = field.get(obj);
				if (value == null) {
					throw new ValidatorException(
							"Null value was found for one of the supplied object fields. Object class is "
									+ obj.getClass().getName());
				}
			} catch (IllegalAccessException e) {
				log.error("Exception:", e);
			}
		}

	}

	public void validateCountry(String name) throws ValidatorException {
		boolean found = false;

		List<Country> list = countryReader.getCountries();
		for (int i = 0, n = countryReader.getCountries().size(); i < n; i++) {
			if (name.equals(list.get(i).getName())) {
				found = true;
				break;
			}
		}

		if (!found) {
			throw new ValidatorException("Country: " + name + "not found.");
		}
	}

	public void validateNString(int min, int max, String type, String toValidate)
			throws ValidatorException {
		boolean isNumber = false;
		isNumber = org.apache.commons.validator.GenericValidator
				.isLong(toValidate);

		if (toValidate.length() < min || toValidate.length() > max || !isNumber) {
			throw new ValidatorException("Invalid NString for " + type
					+ " string: " + toValidate);
		}
	}

	public void validateAString(int min, int max, String type, String toValidate)
			throws ValidatorException {
		if (toValidate.length() < min || toValidate.length() > max) {
			throw new ValidatorException("Invalid length for " + type
					+ " string: " + toValidate);
		}
	}

	public void validateNumber(long min, long max, String type, Long number)
			throws ValidatorException {
		if (number == null || number > max || number < min) {
			throw new ValidatorException("Invalid value for " + type
					+ ". Value = " + number + ". Should be between " + min
					+ " and " + max);
		}
	}

	public void validatePassword(NewCustomerInput input)
			throws ValidatorException {
		if (!input.getBusinessName().toLowerCase().equals(input.getPassword())) {
			throw new ValidatorException("Invalid password: "
					+ input.getPassword() + " (businessName: "
					+ input.getBusinessName());
		}
	}

	public void validatePaymentMethod(String method) throws ValidatorException {
		if (!method.equals("PO") && !method.equals("CC")) {
			throw new ValidatorException(
					"Payment method is not PO or CC. paymentMethod: " + method);
		}
	}

	public void validateEmailFormat(NewCustomerInput input)
			throws ValidatorException {
		validateEmail(input.getContactEmail());
		if (!input.getContactEmail().equals(
				input.getContactFName() + "@" + input.getBusinessName()
						+ ".com")) {
			throw new ValidatorException("Wrong email:"
					+ input.getContactEmail() + ". Should be"
					+ input.getContactFName() + "@" + input.getBusinessName()
					+ ".com");
		}
	}

	public void validateEmail(String email) throws ValidatorException {
		boolean validEmail = false;

		if (email.length() > 50) {
			throw new ValidatorException(
					"Invalid size for Email. Should be 50 chars max. Email: "
							+ email);
		}

		validEmail = EmailValidator.getInstance().isValid(email);
		if (!validEmail) {
			throw new ValidatorException("Invalid Email: " + email);
		}
	}

	public void setCountryReader(CountryReader countryReader) {
		this.countryReader = countryReader;
	}
}
